// hlavickove subory *******************************************************
// systemove kniznice
#include <math.h>

// vlastny .h subor
#include "m3dmath.h"

// konstruktor *************************************************************
m3dmath::m3dmath()
{
 // priesecnik
 ipoint.set(0.0f,0.0f,0.0f);

 // vysledky
 vector_result.set(0.0f,0.0f,0.0f);
 scalar_result=0.0f;
 recip=0.0f;

 // velkost
 magnitude=0.0f;

 // pomocne vektory
 v.set(0.0f,0.0f,0.0f);
 vA.set(0.0f,0.0f,0.0f);
 vB.set(0.0f,0.0f,0.0f);
 // pomocne premenne
 d=0.0f; m=0.0f;
}

// destruktor **************************************************************
m3dmath::~m3dmath()
{

}

// absolutna hodnotu skalara ***********************************************
float m3dmath::myAbs(float N)
{
 if (N<0.0f) return(-N);

 return(N);
}

// vypocet prvkov 3-rozmerneho vektora *************************************
tVec3 m3dmath::Vector(tVec3 P1,tVec3 P2)
{
 vector_result.x=P2.x-P1.x;
 vector_result.y=P2.y-P1.y;
 vector_result.z=P2.z-P1.z;

 return(vector_result); // vysledok = P2 - P1
}

// sucet vektorov **********************************************************
tVec3 m3dmath::AddVector(tVec3 V1,tVec3 V2)
{
 vector_result.x=V1.x+V2.x;
 vector_result.y=V1.y+V2.y;
 vector_result.z=V1.z+V2.z;

 return(vector_result); // vysledok = V1 + V2
}

// delenie vektora skalarom ************************************************
tVec3 m3dmath::DivVectorByScalar(tVec3 V,float S)
{
 if (S==0.0f) S=1.0f;
 recip=1.0f/S;

 vector_result.x=V.x*recip;
 vector_result.y=V.y*recip;
 vector_result.z=V.z*recip;

 return(vector_result); // vysledok = V / S
}

// skalarny sucin dvoch vektorov *******************************************
float m3dmath::Dot(tVec3 V1,tVec3 V2)
{
 scalar_result=(V1.x*V2.x)+(V1.y*V2.y)+(V1.z*V2.z);

 return(scalar_result); // vysledok = V1 . V2
}

// vektorovy sucin dvoch vektorov ******************************************
tVec3 m3dmath::Cross(tVec3 V1,tVec3 V2)
{
 vector_result.x=((V1.y*V2.z)-(V1.z*V2.y));
 vector_result.y=((V1.z*V2.x)-(V1.x*V2.z));
 vector_result.z=((V1.x*V2.y)-(V1.y*V2.x));

 return(vector_result); // vysledok = V1 x V2
}

// normalizacia vektora ****************************************************
tVec3 m3dmath::Normalize(tVec3 N)
{
 magnitude=(float) (sqrt(N.x*N.x+N.y*N.y+N.z*N.z));

 vector_result=DivVectorByScalar(N,magnitude);

 return(vector_result); // vysledok = N / magnitude
}

// vzdialenost dvoch bodov *************************************************
float m3dmath::Distance(tVec3 P1,tVec3 P2)
{
 v=P2-P1;
 scalar_result=(float) (sqrt(v.x*v.x+v.y*v.y+v.z*v.z));

 return(scalar_result); // vysledok = vzdialenost P1 - P2
}

// kolizia luca s trojuholnikom ********************************************
bool m3dmath::RayTriCollision(tVec3 P,tVec3 L,tVec3 N,float D,
                              tVec4 C1,tVec4 C2,tVec4 C3)
{
 // vektor P -> L (bod -> poloha svetelneho zdroja)
 vA=L-P; vA=Normalize(vA);

 // skalarny sucin normaly trojuholnika a luca
 d=Dot(N,vA);

 // v pripade, ak je luc kolmy na normalu trojuholnika, nedoslo ku kolizii
 if (d==0.0f) return(false);

 // projekcia bodu do roviny tvorenej trojuholnikom
 m=-(N.x*P.x+N.y*P.y+N.z*P.z+D)/d;

 // ak je rovina, ktoru tvori trojuholnik pred bodom, je potrebne otestovat,
 // ci priesecnik luca a roviny lezi v trojuholniku
 if (m>0.0f) {
  // priesecnik luca a roviny tvorenej trojuholnikom
  ipoint=P+vA*m;

  // ak priesecnik lezi za stenou trojuholnika, nelezi v trojuholniku
  if (ipoint.x*C1.x+ipoint.y*C1.y+ipoint.z*C1.z+C1.w>0.0f) return(false);
  if (ipoint.x*C2.x+ipoint.y*C2.y+ipoint.z*C2.z+C2.w>0.0f) return(false);
  if (ipoint.x*C3.x+ipoint.y*C3.y+ipoint.z*C3.z+C3.w>0.0f) return(false);

  // priesecnik lezi v trojuholniku => doslo ku kolizii
  return(true);
 }

 // nedoslo ku kolizii
 return(false);
}

// vrati bod na usecke P1-P2, ktory je najblizsie k bodu P3 *****************
tVec3 m3dmath::PointOnLine(tVec3 P1,tVec3 P2,tVec3 P3)
{
 vA=P3-P1;                   // 1. vektor
 vB=P2-P1; vB=Normalize(vB); // 2. vektor

 // projekcia bodu do usecky P1-P2
 // vysledok skalarneho sucinu moze byt:
 // <=0 - bod lezi pred bodom P1
 // >=d - bod lezi za bodom P2
 // >0 a <d - bod lezi na usecke P1-P2
 m=Dot(vA,vB);

 // bod lezi pred bodom P1, sice nepatri usecke,
 // ale funkcia vrati bod P1
 if (m<=0.0f) return P1;

 // vypocet vzdialenosti P1-P2
 d=Distance(P1,P2);

 // bod lezi za bodom P2, sice nepatri usecke,
 // ale funkcia vrati bod P2
 if (m>=d) return P2;

 // vypocet polohy bodu, ktory lezi na usecke P1-P2
 vector_result=P1+vB*m;

 return(vector_result);
}

// test vzajomnej polohy bodu a trojuholnika *******************************
bool m3dmath::PointInTriangle(tVec3 IP,tVec4 C1,tVec4 C2,tVec4 C3)
{
 // ak bod lezi za stenou trojuholnika, nelezi v trojuholniku
 if (IP.x*C1.x+IP.y*C1.y+IP.z*C1.z+C1.w>0.0f) return(false);
 if (IP.x*C2.x+IP.y*C2.y+IP.z*C2.z+C2.w>0.0f) return(false);
 if (IP.x*C3.x+IP.y*C3.y+IP.z*C3.z+C3.w>0.0f) return(false);

 // bod lezi pred vsetkymi troma stenami, lezi v trojuholniku
 return(true);
}

// test vzajomnej polohy gule a trojuholnika *******************************
bool m3dmath::SphereInTriangle(tVec4 T,tVec3 C,float R)
{
 // vypocet vzdialenosti stredu gule od trojuholnika (roviny, ktoru tvori)
 // d=abs(AX+BY+CZ+D)
 d=myAbs(T.x*C.x+T.y*C.y+T.z*C.z+T.w);

 // ak je vzdialenost gule od roviny vacsia ako jej polomer, nedoslo ku kolizii
 if (myAbs(d)>=R) return(false);

 // INFO: vzdialenost stredu gule od roviny
 //fprintf(stderr,"%f\n",d);

 // projekcia stredu gule do roviny, ktoru tvori trojuholnik
 ipoint.x=C.x-(T.x*d);
 ipoint.y=C.y-(T.y*d);
 ipoint.z=C.z-(T.z*d);

 // gula kolidovala s trojuholnikom (rovinou, ktoru tvori)
 return(true);
}

// test vzajomnej polohy gule a hran trojuholnika **************************
bool m3dmath::SphereInEdge(tVec3 T1,tVec3 T2,tVec3 T3,tVec3 C,float R)
{
 // 1. hrana trojuholnika
 ipoint=PointOnLine(T1,T2,C);
 if (Distance(ipoint,C)<R) return(true);
 // 2. hrana trojuholnika
 ipoint=PointOnLine(T2,T3,C);
 if (Distance(ipoint,C)<R) return(true);
 // 3. hrana trojuholnika
 ipoint=PointOnLine(T3,T1,C);
 if (Distance(ipoint,C)<R) return(true);

 //nedoslo ku kolizii
 return(false);
}
