#ifndef _M3DMATH_H_
#define _M3DMATH_H_

#define PI    3.141592653589f
#define PI180 0.017453292519f
#define PI2   1.570796327f
#define DEG2RAD(a) (PI/180*(a))
#define RAD2DEG(a) (180/PI*(a))
#define SQR(a) ((a)*(a))

// 2-rozmerny vektor
struct tVec2 {
 float x,y;
};

// 3-rozmerny vektor
class tVec3 {
 public:
  union {
   struct {
    float x,y,z;
   };
   float v[3];
  };

  // konstruktory
  tVec3(float x=0.0f,float y=0.0f,float z=0.0f):x(x),y(y),z(z) {}
  tVec3(const tVec3 &vec):x(vec.x),y(vec.y),z(vec.z) {}
  tVec3(const tVec3 &a,const tVec3 &b):x(b.x-a.x),y(b.y-a.y),z(b.z-a.z) {}

  // priradenie hodnot prvkom vektora
  void set(float SX,float SY,float SZ) {
   x=SX; y=SY; z=SZ;
  }
  // priradenie hodnot prvkom vektora
  const tVec3 &operator=(const tVec3 &VEC) {
   x=VEC.x; y=VEC.y; z=VEC.z;
   return *this;
  }
  // rovnost vektorov
  const bool operator==(const tVec3 &VEC) const {
   return ((x==VEC.x)&&(y==VEC.y)&&(z==VEC.z));
  }
  // nerovnost vektorov
  const bool operator!=(const tVec3 &VEC) const {
   return !(*this==VEC);
  }
  // index
  float &operator[](const long IDX) {
   return *((&x)+IDX);
  }
  // scitanie vektorov
  const tVec3 operator+(const tVec3 &VEC) const {
   return tVec3(x+VEC.x,y+VEC.y,z+VEC.z);
  }
  // negacia kladneho vektora
  const tVec3 operator+() const {
   return tVec3(*this);
  }
  // pricitanie k vektoru
  const tVec3 &operator+=(const tVec3 &VEC) {
   x+=VEC.x; y+=VEC.y; z+=VEC.z;
   return *this;
  }
  // odcitanie vektorov
  const tVec3 operator-(const tVec3 &VEC) const {
   return tVec3(x-VEC.x,y-VEC.y,z-VEC.z);
  }
  // negacia zaporneho vektora
  const tVec3 operator-() const {
   return tVec3(-x,-y,-z);
  }
  // odcitanie od vektora
  const tVec3 &operator-=(const tVec3 &VEC) {
   x-=VEC.x; y-=VEC.y; z-=VEC.z;
   return *this;
  }
  // nasobenie skalarom
  const tVec3 &operator*=(const float S) {
   x*=S; y*=S; z*=S;
   return *this;
  }
  // delenie skalarom
  const tVec3 &operator/=(const float S) {
   const float recip=1.0f/S;
   x*=recip; y*=recip; z*=recip;
   return *this;
  }
  // nasobenie skalarom
  const tVec3 operator*(const float S) const {
   return tVec3(x*S,y*S,z*S);
  }
  // nasobenie skalarom
  friend const tVec3 operator*(const float S,const tVec3 &VEC) {
   return VEC*S;
  }
  // delenie skalarom
  const tVec3 operator/(const float S) const {
   const float recip=1.0f/S;
   return tVec3(recip*x,recip*y,recip*z);
  }
  // nasobenie vektorov
  const tVec3 operator*(const tVec3 &VEC) const {
   return tVec3(x*VEC.x,y*VEC.y,z*VEC.z);
  }
};

// 4-rozmerny vektor
struct tVec4 {
 float x,y,z,w;
};

// vrchol + texturovacia suradnica
struct tMatVT {
 float x,y,z;    // suradnice vrcholu
 float u,v;      // texturovacie suradnice
 unsigned int s; // vrchol bude/nebude vyuzity
};

// trieda premennych a funkcii (objektov) pre pracu s matematickymi vypoctami
class m3dmath {
public:
 m3dmath();          // konstruktor
 virtual ~m3dmath(); // destruktor

 // absolutna hodnotu skalara
 float myAbs(float N);
 // vypocet prvkov 3-rozmerneho vektora
 tVec3 Vector(tVec3 P1,tVec3 P2);
 // sucet vektorov
 tVec3 AddVector(tVec3 V1,tVec3 V2);
 // delenie vektora skalarom
 tVec3 DivVectorByScalar(tVec3 V,float S);
 // skalarny sucin dvoch vektorov
 float Dot(tVec3 V1,tVec3 V2);
 // vektorovy sucin dvoch vektorov
 tVec3 Cross(tVec3 V1,tVec3 V2);
 // normalizacia vektora
 tVec3 Normalize(tVec3 N);
 // vzdialenost dvoch bodov
 float Distance(tVec3 P1,tVec3 P2);

 // kolizia luca s trojuholnikom
 bool RayTriCollision(tVec3 P,tVec3 L,tVec3 N,float D,
                      tVec4 C1,tVec4 C2,tVec4 C3);
 // vrati bod na usecke P1-P2, ktory je najblizsie k bodu P3
 tVec3 PointOnLine(tVec3 P1,tVec3 P2,tVec3 P3);
 // test vzajomnej polohy bodu a trojuholnika
 bool PointInTriangle(tVec3 IP,tVec4 C1,tVec4 C2,tVec4 C3);
 // test vzajomnej polohy gule a trojuholnika
 bool SphereInTriangle(tVec4 T,tVec3 C,float R);
 // test vzajomnej polohy gule a hran trojuholnika
 bool SphereInEdge(tVec3 T1,tVec3 T2,tVec3 T3,tVec3 C,float R);

 tVec3 ipoint; // priesecnik
private:
 tVec3 vector_result; // vektor
 float scalar_result; // skalar
 float recip;         // reciprocna hodnota skalara

 float magnitude; // velkost

 tVec3 v,vA,vB; // pomocne vektory
 float d,m;     // pomocne premenne
};

#endif
