// hlavickove subory *******************************************************
// systemove kniznice
#include <stdio.h>
#include <stdlib.h>
// GL a GLEXT kniznica
#include <GL/gl.h>
#include <GL/glext.h>

// vlastny .h subor
#include "mFrontend.h"

// konstruktor *************************************************************
mTriangle::mTriangle()
{
 // nastavenie pociatocnych hodnot jednotlivym premennym
 first_tri=NULL; last_tri=NULL;
 tmp_tri=NULL; cur_tri=NULL;
 // celkovy pocet trojuholnikov
 tri_num=0;

 // pole aktivnych trojuholnikov
 act_tri=NULL;
 // pocet aktivnych trojuholnikov
 act_tri_num=0;

 // pole koliznych trojuholnikov
 col_tri=NULL;
 // pocet koliznych trojuholnikov
 col_tri_num=0;

 // pocet zobrazenych trojuholnikov
 render_tri=0;

 // pomocne premenne
 t_a=0; t_b=0; t_c=0;

 // pomocne vektory
 vec0.set(0.0f,0.0f,0.0f);
 vec1.set(0.0f,0.0f,0.0f);
 vec2.set(0.0f,0.0f,0.0f);
 lp.set(0.0f,0.0f,0.0f);
 t0.x=0.0f; t0.y=0.0f;
 t1.x=0.0f; t1.y=0.0f;
 t2.x=0.0f; t2.y=0.0f;

 // identifikator aktualnej textury
 act_tex_id=0;

 // 3D suradnice bodu patriaceho lightmape -lumelu
 lm_position.set(0.0f,0.0f,0.0f);
 // pomocne premenne pre vypocet 3D suradnic lumelov
 dpdu=0.0f; dpdv=0.0f; dqdu=0.0f; dqdv=0.0f; den=0.0f;
 aa=0.0f; bb=0.0f; cc=0.0f;
}

// destruktor **************************************************************
mTriangle::~mTriangle()
{
 // vymazanie zoznamu trojuholnikov
 tri_delall();

 // vymazanie pola aktivnych trojuholnikov
 if (act_tri!=NULL) { free(act_tri); act_tri=NULL; act_tri_num=0; }

 // vymazanie pola koliznych trojuholnikov
 if (col_tri!=NULL) { free(col_tri); col_tri=NULL; col_tri_num=0; }
}

// pridanie noveho trojuholnika ********************************************
int mTriangle::tri_add(tVec3 V0,tVec3 V1,tVec3 V2,
                       tVec2 T0,tVec2 T1,tVec2 T2,
                       GLuint TEX1,GLuint TEX2,
                       tVec4 C,
                       tVec3 N0,tVec3 N1,tVec3 N2,
                       GLuint TYPE)
{
 if (first_tri==NULL) { // vytvorenie 1. trojuholnika
  first_tri=new _tri;
  if (first_tri==NULL) {
   fprintf(stderr,"Chyba vyhradenia pamate pre prvy trojuholnik!\n");
   return(1);
  }
  last_tri=first_tri;
 }
 else {                 // vytvorenie dalsieho trojuholnika
  last_tri->next=new _tri;
  if (last_tri->next==NULL) {
   fprintf(stderr,"Chyba vyhradenia pamate pre novy trojuholnik!\n");
   return(1);
  }
  last_tri=last_tri->next;
 }

 // identifikator trojuholnika
 last_tri->id=tri_num;
 // zvysenie celkoveho poctu trojuholnikov
 tri_num++;

 // vrcholy trojuholnika
 last_tri->v0=V0; last_tri->v1=V1; last_tri->v2=V2;
 // texturovacie suradnice
 last_tri->t0=T0; last_tri->t1=T1; last_tri->t2=T2;
 // identifikatory textur
 last_tri->tex1id=TEX1;
 last_tri->tex2id=TEX2;
 // identifikator lightmap textury - priradi sa neskor
 last_tri->LMtexid=0;
 // farba
 last_tri->c=C;
 // normaly vo vrcholoch - vypocitaju sa neskor
 last_tri->n0=N0; last_tri->n1=N1; last_tri->n2=N2;
 // prvok D rovnice roviny, ktoru tvori trojuholnik - vypocita sa neskor
 last_tri->d=0.0f;
 // kolizne normaly +D jednotlivych stien pre vypocet kolizie - vypocitaju sa neskor
 last_tri->cn0.x=0.0f; last_tri->cn0.y=0.0f; last_tri->cn0.z=0.0f; last_tri->cn0.w=0.0f;
 last_tri->cn1.x=0.0f; last_tri->cn1.y=0.0f; last_tri->cn1.z=0.0f; last_tri->cn1.w=0.0f;
 last_tri->cn2.x=0.0f; last_tri->cn2.y=0.0f; last_tri->cn2.z=0.0f; last_tri->cn2.w=0.0f;
 // minimalne a maximalne suradnice vrcholov na danych osiach
 last_tri->minx=last_tri->v0.x;
 if (last_tri->v1.x<last_tri->minx) last_tri->minx=last_tri->v1.x;
 if (last_tri->v2.x<last_tri->minx) last_tri->minx=last_tri->v2.x;
 last_tri->maxx=last_tri->v0.x;
 if (last_tri->v1.x>last_tri->maxx) last_tri->maxx=last_tri->v1.x;
 if (last_tri->v2.x>last_tri->maxx) last_tri->maxx=last_tri->v2.x;

 last_tri->miny=last_tri->v0.y;
 if (last_tri->v1.y<last_tri->miny) last_tri->miny=last_tri->v1.y;
 if (last_tri->v2.y<last_tri->miny) last_tri->miny=last_tri->v2.y;
 last_tri->maxy=last_tri->v0.y;
 if (last_tri->v1.y>last_tri->maxy) last_tri->maxy=last_tri->v1.y;
 if (last_tri->v2.y>last_tri->maxy) last_tri->maxy=last_tri->v2.y;

 last_tri->minz=last_tri->v0.z;
 if (last_tri->v1.z<last_tri->minz) last_tri->minz=last_tri->v1.z;
 if (last_tri->v2.z<last_tri->minz) last_tri->minz=last_tri->v2.z;
 last_tri->maxz=last_tri->v0.z;
 if (last_tri->v1.z>last_tri->maxz) last_tri->maxz=last_tri->v1.z;
 if (last_tri->v2.z>last_tri->maxz) last_tri->maxz=last_tri->v2.z;
 // indikator spracovania
 last_tri->finish_bit=false;
 // pomocny indikator
 last_tri->select=false;
 // typ trojuholnika
 last_tri->type=TYPE;

 // odkaz na nasledujuci trojuholnik
 last_tri->next=NULL;

 // vypocet normal v jednotlivych vrcholoch trojuholnika
 tri_normals();

 // vypocet koliznych normal stien trojuholnika potrebnych pre neskorsi
 // vypocet kolizie s tymto trojuholnikom
 tri_collision_normals();

 return(0);
}

// vypocet normal v jednotlivych vrcholoch trojuholnika ********************
void mTriangle::tri_normals(void)
{
 // normala v 1. vrchole
 vec0=last_tri->v1-last_tri->v0; // 1. vektor
 vec1=last_tri->v2-last_tri->v0; // 2. vektor

 // vektorovy sucin vektorov = normala
 vec2=mm.Cross(vec0,vec1); vec2=mm.Normalize(vec2);

 last_tri->n0=vec2;

 // normala v 2. vrchole
 vec0=last_tri->v2-last_tri->v1; // 1. vektor
 vec1=last_tri->v0-last_tri->v1; // 2. vektor

 // vektorovy sucin vektorov = normala
 vec2=mm.Cross(vec0,vec1); vec2=mm.Normalize(vec2);

 last_tri->n1=vec2;

 // normala v 3. vrchole
 vec0=last_tri->v0-last_tri->v2; // 1. vektor
 vec1=last_tri->v1-last_tri->v2; // 2. vektor

 // vektorovy sucin vektorov = normala
 vec2=mm.Cross(vec0,vec1); vec2=mm.Normalize(vec2);

 last_tri->n2=vec2;

 // vypocet prvku D rovnice roviny, ktoru tvori trojuholnik
 // D=-(AX+BY+CZ)
 // A,B,C - normala trojuholnika
 // X,Y,Z - lubovolny vrchol trojuholnika
 last_tri->d=-(last_tri->n0.x*last_tri->v0.x+
               last_tri->n0.y*last_tri->v0.y+
               last_tri->n0.z*last_tri->v0.z);
}

// vypocet koliznych normal +D jednotlivych stien **************************
void mTriangle::tri_collision_normals(void)
{
 // kolizna normala pre 1. stenu trojuholnika
 vec0=last_tri->v1-last_tri->v0; // 1. vektor
 vec1=last_tri->n0;              // 2. vektor

 // vektorovy sucin vektorov = kolizna normala
 // prvky koliznej normaly su prvkami rovnice roviny
 // A,B,C, prvok D sa dopocita neskor
 vec2=mm.Cross(vec0,vec1); vec2=mm.Normalize(vec2);

 last_tri->cn0.x=vec2.x;
 last_tri->cn0.y=vec2.y;
 last_tri->cn0.z=vec2.z;
 // z rovnice roviny AX+BY+CZ+D=0 => D=-(AX+BY+CZ)
 last_tri->cn0.w=-(vec2.x*last_tri->v0.x+vec2.y*last_tri->v0.y+vec2.z*last_tri->v0.z);

 // kolizna normala pre 2. stenu trojuholnika
 vec0=last_tri->v2-last_tri->v1; // 1. vektor
 vec1=last_tri->n1;              // 2. vektor

 // vektorovy sucin vektorov = kolizna normala
 // prvky koliznej normaly su prvkami rovnice roviny
 // A,B,C, prvok D sa dopocita neskor
 vec2=mm.Cross(vec0,vec1); vec2=mm.Normalize(vec2);

 last_tri->cn1.x=vec2.x;
 last_tri->cn1.y=vec2.y;
 last_tri->cn1.z=vec2.z;
 // z rovnice roviny AX+BY+CZ+D=0 => D=-(AX+BY+CZ)
 last_tri->cn1.w=-(vec2.x*last_tri->v1.x+vec2.y*last_tri->v1.y+vec2.z*last_tri->v1.z);

 // kolizna normala pre 3. stenu trojuholnika
 vec0=last_tri->v0-last_tri->v2; // 1. vektor
 vec1=last_tri->n2;              // 2. vektor

 // vektorovy sucin vektorov = kolizna normala
 // prvky koliznej normaly su prvkami rovnice roviny
 // A,B,C, prvok D sa dopocita neskor
 vec2=mm.Cross(vec0,vec1); vec2=mm.Normalize(vec2);

 last_tri->cn2.x=vec2.x;
 last_tri->cn2.y=vec2.y;
 last_tri->cn2.z=vec2.z;
 // z rovnice roviny AX+BY+CZ+D=0 => D=-(AX+BY+CZ)
 last_tri->cn2.w=-(vec2.x*last_tri->v2.x+vec2.y*last_tri->v2.y+vec2.z*last_tri->v2.z);
}

// priprava poli pre aktivne a kolizne trojuholniky ************************
int mTriangle::tri_finish(GLuint N)
{
 // priprava pola takej velkosti, aby don bolo mozne ulozit
 // vsetky trojuholniky pouziteho stromu, avsak z tohto pola bude pouzita
 // iba cast, ktora sa naplni odkazmi na aktivne trojuholniky
 act_tri=(_tri **) malloc(N*sizeof(_tri));
 if (act_tri==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre aktivne trojuholniky!\n");
  return(1);
 }

 // priprava pola takej velkosti, aby don bolo mozne ulozit
 // vsetky trojuholniky pouziteho stromu, avsak z tohto pola bude pouzita
 // iba cast, ktora sa naplni odkazmi na kolizne trojuholniky
 // pole mozno zmensit na 1/2 nakolko mozeme pocitat, ze avatar
 // nikdy nebude kolidovat naraz so vsetkymi trojuholnikmi sceny
 // pozn.: toto pole mozno este zmensit, avsak pretoze sa
 // oktanovy strom vytvara dynamicky, je tazke definovat kolko
 // trojuholnikov sa z neho v danom okamihu vyberie
 col_tri=(_tri **) malloc(N/2*sizeof(_tri));
 if (col_tri==NULL) {
  fprintf(stderr,"Chyba vyhradenia pamate pre kolizne trojuholniky!\n");
  return(1);
 }

 return(0);
}

// vykreslenie vsetkych trojuholnikov ulozenych v zozname trojuholnikov ****
int mTriangle::tri_draw()//mFrustum *F)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING); // vypnutie osvetlenia

 // prvy trojuholnik v zozname
 tmp_tri=first_tri; if (tmp_tri==NULL) return(1);

 // identifikator aktualnej textury
 // k vymene textur dojde iba v pripade, ak je to potrebne,
 // inak zostane aktivna povodna textura
 act_tex_id=0;

 // 1. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glEnable(GL_TEXTURE_2D);
 // 2. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glEnable(GL_TEXTURE_2D);
 // kombinacna funkcia
 glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_EXT);
 glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB_EXT,GL_MODULATE);

 // pre vsetky trojuholniky v zozname
 while (tmp_tri!=NULL) {

  // Frustum Culling
//  if (F->CubeInFrustum2(tmp_tri->minx,tmp_tri->maxx,
//                        tmp_tri->miny,tmp_tri->maxy,
//                        tmp_tri->minz,tmp_tri->maxz)) {

   // ak je potrebne vymenit aktualnu texturu
   //if (tmp_tri->tex1id!=act_tex_id) {
    // identifikator novej aktualnej textury
    act_tex_id=tmp_tri->tex1id;
    // pouzije sa multitexturing
    // 1. textura - original
    // 2. textura - lightmapa
    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D,tmp_tri->tex1id);
    glActiveTextureARB(GL_TEXTURE1_ARB);
    glBindTexture(GL_TEXTURE_2D,tmp_tri->LMtexid);
   //} // vymena aktualnej textury

   // vykreslenie trojuholnikov
   glBegin(GL_TRIANGLES);
    // farba
    glColor4f(tmp_tri->c.x,tmp_tri->c.y,tmp_tri->c.z,tmp_tri->c.w);
    // texturovacie suradnice, normala, vrchol
    glMultiTexCoord2fARB(GL_TEXTURE0_ARB,tmp_tri->t0.x,tmp_tri->t0.y);
    glMultiTexCoord2fARB(GL_TEXTURE1_ARB,tmp_tri->t0.x,tmp_tri->t0.y);
    //glTexCoord2f(tmp_tri->t0.x,tmp_tri->t0.y); // standardne texturovanie
    glNormal3f(tmp_tri->n0.x,tmp_tri->n0.y,tmp_tri->n0.z);
    glVertex3f(tmp_tri->v0.x,tmp_tri->v0.y,tmp_tri->v0.z);
    // texturovacie suradnice, normala, vrchol
    glMultiTexCoord2fARB(GL_TEXTURE0_ARB,tmp_tri->t1.x,tmp_tri->t1.y);
    glMultiTexCoord2fARB(GL_TEXTURE1_ARB,tmp_tri->t1.x,tmp_tri->t1.y);
    //glTexCoord2f(tmp_tri->t1.x,tmp_tri->t1.y); // standardne texturovanie
    glNormal3f(tmp_tri->n1.x,tmp_tri->n1.y,tmp_tri->n1.z);
    glVertex3f(tmp_tri->v1.x,tmp_tri->v1.y,tmp_tri->v1.z);
    // texturovacie suradnice, normala, vrchol
    glMultiTexCoord2fARB(GL_TEXTURE0_ARB,tmp_tri->t2.x,tmp_tri->t2.y);
    glMultiTexCoord2fARB(GL_TEXTURE1_ARB,tmp_tri->t2.x,tmp_tri->t2.y);
    //glTexCoord2f(tmp_tri->t2.x,tmp_tri->t2.y); // standardne texturovanie
    glNormal3f(tmp_tri->n2.x,tmp_tri->n2.y,tmp_tri->n2.z);
    glVertex3f(tmp_tri->v2.x,tmp_tri->v2.y,tmp_tri->v2.z);
   glEnd();

   // standardna farba
   glColor4f(1.0f,1.0f,1.0f,1.0f);

//  } // Frustum Culling

  // vyber dalsieho trojuholnika zo zoznamu
  tmp_tri=tmp_tri->next;

 } // pre vsetky trojuholniky v zozname

 // 2. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_2D);
 // 1. texturovacia jednotka
 glActiveTextureARB(GL_TEXTURE0_ARB);

 // vyber ulozenych atributov - LIGHTING, TEXTURE
 glPopAttrib();

 return(0);
}

// vymazanie zoznamu trojuholnikov *****************************************
int mTriangle::tri_delall(void)
{
 if (first_tri!=NULL) {
  while (first_tri->next!=NULL) { // zmazanie vsetkych trojuholnikov zo zoznamu
   tmp_tri=first_tri->next;
   delete first_tri;
   first_tri=tmp_tri;
  }
  delete first_tri;               // zmazanie 1. trojuholnika
  first_tri=NULL;
 }
 last_tri=NULL; tmp_tri=NULL; cur_tri=NULL;

 tri_num=0; // celkovy pocet trojuholnikov

 return(0);
}

// vypocet lightmap pre vsetky trojuholniky v zozname trojuholnikov ********
int mTriangle::tri_lightmaps(GLfloat LX,GLfloat LY,GLfloat LZ)
{
 // vymazanie skor vytvorenych objektov lightmap
 // takyto pristup nie je najvhodnejsi, ale na ukazku postacuje
 lmTexture.DeleteLightmaps();

 // prvy trojuholnik v zozname
 tmp_tri=first_tri; if (tmp_tri==NULL) return(1);
 cur_tri=tmp_tri;

 // pre vsetky trojuholniky v zozname
 while (cur_tri!=NULL) {

  // nastavenie neutralnej farby lumelov
  // neutralna farba = 128
  for (t_b=0;t_b<LM_SIZE;t_b++) {
   for (t_a=0;t_a<LM_SIZE;t_a++) {
    // index v poli farieb lumelov
    t_c=3*t_a+3*t_b*LM_SIZE;
    lm_color[t_c+0]=128; // red
    lm_color[t_c+1]=128; // green
    lm_color[t_c+2]=128; // blue
   }
  }

  // nastavenie farby lumelov
  // lumel v tieni   = 0
  // osvetleny lumel = 255
  for (t_b=0;t_b<LM_SIZE;t_b++) {
   for (t_a=0;t_a<LM_SIZE;t_a++) {
    // index v poli farieb lumelov
    t_c=3*t_a+3*t_b*LM_SIZE;

    // 2D suradnice lumelu
    lm_position.x=(((GLfloat) t_a)+0.5f)/LM_SIZE;
    lm_position.y=(((GLfloat) t_b)+0.5f)/LM_SIZE;

    // INFO: 2D suradnice lumelu
    //fprintf(stderr,"U:%f V:%f\n",lm_position.x,lm_position.y);

    // vypocet 3D suradnic lumelu
    lm_worldpos(&lm_position,cur_tri->v0,cur_tri->v1,cur_tri->v2,
                             cur_tri->t0,cur_tri->t1,cur_tri->t2,
                             cur_tri->n0,cur_tri->d);

    // INFO: 3D suradnice lumelu
    //fprintf(stderr,"x:%f y:%f z:%f\n",lm_position.x,lm_position.y,lm_position.z);

    // poloha svetelneho zdroja
    lp.x=LX; lp.y=LY; lp.z=LZ;

    // vektor poloha svetelneho zdroja -> poloha lumelu
    vec0=lm_position-lp;

    // ak je svetelny zdroj privrateny k trojuholniku
    // v takom pripade moze tento trojuholnik osvetlit
    if (mm.Dot(vec0,cur_tri->n0)<0.0f) {

     // detekcia kolizie luca a vsetkych ostatnych trojuholnikov
     tmp_tri=first_tri;
     while (tmp_tri!=NULL) {
      // detekcia sa vykona iba so vsetkymi ostatnymi trojuholnikmi,
      // nie s tym istym trojuholnikom
      if (tmp_tri->id!=cur_tri->id) {
       // kolizia luca s trojuholnikom
       if (mm.RayTriCollision(lm_position,lp,tmp_tri->n0,tmp_tri->d,
                              tmp_tri->cn0,tmp_tri->cn1,tmp_tri->cn2)) {
         // nastala kolizia -> lumel je v tieni
         lm_color[t_c+0]=0; lm_color[t_c+1]=0; lm_color[t_c+2]=0;
       }
       else {
        // ak nema lumel neutralnu farbu, nemoze byt osvetleny,
        // zabrani sa tak prepisaniu farby lumelu v pripade
        // ak uz je v tieni co mohlo nastat pri testoch
        // s predoslymi trojuholnikmi
        if (lm_color[t_c+0]==128) {
         // nenastala kolizia -> lumel je osvetleny
         lm_color[t_c+0]=255; lm_color[t_c+1]=255; lm_color[t_c+2]=255;
        }
       }
      }
      // vyber dalsieho trojuholnika zo zoznamu
      tmp_tri=tmp_tri->next;
     } // pre vsetky ostatne trojuholniky

    }
    else {
     // svetelny zdroj je odvrateny od trojuholnika
     // v takomto pripade nie je trojuholnik osvetleny
     lm_color[t_c+0]=0; lm_color[t_c+1]=0; lm_color[t_c+2]=0;
    }

   } // pre vsetky lumely v smere U (t_a)
  } // pre vsetky lumely v smere V (t_b)

  // rozmazanie lightmapy - blur
  // ziskame tak efekt rozmazanyn okrajov tienov
  for (t_b=1;t_b<LM_SIZE-1;t_b++) {
   for (t_a=1;t_a<LM_SIZE-1;t_a++) {
    // index v poli farieb lumelov
    t_c=3*t_a+3*t_b*LM_SIZE;
    // vypocita sa priemerna hodnota farieb 8-mich okolitych lumelov
    lm_color[t_c+0]=
     (lm_color[3*(t_a-1)+3*(t_b-1)*LM_SIZE+0]+lm_color[3*(t_a+0)+3*(t_b-1)*LM_SIZE+0]+
      lm_color[3*(t_a+1)+3*(t_b-1)*LM_SIZE+0]+lm_color[3*(t_a-1)+3*(t_b+0)*LM_SIZE+0]+
      lm_color[3*(t_a+1)+3*(t_b+0)*LM_SIZE+0]+lm_color[3*(t_a-1)+3*(t_b+1)*LM_SIZE+0]+
      lm_color[3*(t_a+0)+3*(t_b+1)*LM_SIZE+0]+lm_color[3*(t_a+1)+3*(t_b+1)*LM_SIZE+0])/8;
    lm_color[t_c+1]=
     (lm_color[3*(t_a-1)+3*(t_b-1)*LM_SIZE+1]+lm_color[3*(t_a+0)+3*(t_b-1)*LM_SIZE+1]+
      lm_color[3*(t_a+1)+3*(t_b-1)*LM_SIZE+1]+lm_color[3*(t_a-1)+3*(t_b+0)*LM_SIZE+1]+
      lm_color[3*(t_a+1)+3*(t_b+0)*LM_SIZE+1]+lm_color[3*(t_a-1)+3*(t_b+1)*LM_SIZE+1]+
      lm_color[3*(t_a+0)+3*(t_b+1)*LM_SIZE+1]+lm_color[3*(t_a+1)+3*(t_b+1)*LM_SIZE+1])/8;
    lm_color[t_c+2]=
     (lm_color[3*(t_a-1)+3*(t_b-1)*LM_SIZE+2]+lm_color[3*(t_a+0)+3*(t_b-1)*LM_SIZE+2]+
      lm_color[3*(t_a+1)+3*(t_b-1)*LM_SIZE+2]+lm_color[3*(t_a-1)+3*(t_b+0)*LM_SIZE+2]+
      lm_color[3*(t_a+1)+3*(t_b+0)*LM_SIZE+2]+lm_color[3*(t_a-1)+3*(t_b+1)*LM_SIZE+2]+
      lm_color[3*(t_a+0)+3*(t_b+1)*LM_SIZE+2]+lm_color[3*(t_a+1)+3*(t_b+1)*LM_SIZE+2])/8;
   }
  }

  // vytvorenie objektu lightmapy aktualneho trojuholnika
  cur_tri->LMtexid=lmTexture.CreateLightmapTexture(T_DEFAULT,LM_SIZE,lm_color);

  // vyber dalsieho trojuholnika zo zoznamu
  cur_tri=cur_tri->next;
 } // pre vsetky trojuholniky v zozname

 return(0);
}

// vypocet 3D suradnic lumelu **********************************************
int mTriangle::lm_worldpos(tVec3 *LP,tVec3 V0,tVec3 V1,tVec3 V2,
                                     tVec2 T0,tVec2 T1,tVec2 T2,
                                     tVec3 N,float D)
{
 // denominator=(v0-v2)(u1-u2)-(v1-v2)(u0-u2)
 // vypocita sa z texturovacich suradnic U a V
 // pricom U = T.x, V = T.y
 den=(T0.y-T2.y)*(T1.x-T2.x)-(T1.y-T2.y)*(T0.x-T2.x);
 if (den==0.0f) den=1.0f;
 den=1.0f/den; // reciprocna hodnota

 // nasledovne parametre vznikli odvodenim zo vzorcov
 // ziskanych konstrukciou dvoch podobnych trojuholnikov
 // ide o parametre, pomocou ktorych mozno nasledne
 // vypocitat 3D suradnice lumelu, pri tychto vypoctoch
 // dochadza k posunu 3D vrcholu po celej ploche trojuholnika
 // v 3D priestore
 // dpdu=((x1-x2)(v0-v2)-(x0-x2)(v1-v2))/denominator
 // dpdv=((x1-x2)(u0-u2)-(x0-x2)(u1-u2))/-denominator
 // dqdu=((y1-y2)(v0-v2)-(y0-y2)(v1-v2))/denominator
 // dqdv=((y1-y2)(u0-u2)-(y0-y2)(u1-u2))/-denominator

 // projekcia 3D trojuholnika do 2D roviny, na zaklade
 // najvacsieho prvku normaly trojuholnika (roviny)
 // urcime, do ktorej pravouhlej roviny bude trojuholnik
 // projektovany, nasledne z rovnice roviny AX+BY+CZ+D=0
 // ziskame 3. suradnicu:
 // x=-(BY+CZ+D)/A y=-(AX+CZ+D)/B z=-(AX+BY+D)/C
 // pos0=v0.0+(dpdu*LP->x)+(dpdv*LP->y);
 // pos1=v0.1+(dqdu*LP->x)+(dqdv*LP->y);

 // 3D suradnice lumelu
 if ((N.x>=N.y)&&(N.x>=N.z)) {      // projekcia do roviny YZ
  if (N.x==0.0f) N.x=1.0f;

  dpdu= den*((V1.y-V2.y)*(T0.y-T2.y)-(V0.y-V2.y)*(T1.y-T2.y));
  dpdv=-den*((V1.y-V2.y)*(T0.x-T2.x)-(V0.y-V2.y)*(T1.x-T2.x));
  dqdu= den*((V1.z-V2.z)*(T0.y-T2.y)-(V0.z-V2.z)*(T1.y-T2.y));
  dqdv=-den*((V1.z-V2.z)*(T0.x-T2.x)-(V0.z-V2.z)*(T1.x-T2.x));

  bb=V0.y+(dpdu*LP->x)+(dpdv*LP->y);
  cc=V0.z+(dqdu*LP->x)+(dqdv*LP->y);
  aa=-(N.y*bb+N.z*cc+D)/N.x;
 }
 else if ((N.y>=N.x)&&(N.y>=N.z)) { // projekcia do roviny XZ
       if (N.y==0.0f) N.y=1.0f;

       dpdu= den*((V1.x-V2.x)*(T0.y-T2.y)-(V0.x-V2.x)*(T1.y-T2.y));
       dpdv=-den*((V1.x-V2.x)*(T0.x-T2.x)-(V0.x-V2.x)*(T1.x-T2.x));
       dqdu= den*((V1.z-V2.z)*(T0.y-T2.y)-(V0.z-V2.z)*(T1.y-T2.y));
       dqdv=-den*((V1.z-V2.z)*(T0.x-T2.x)-(V0.z-V2.z)*(T1.x-T2.x));

       aa=V0.x+(dpdu*LP->x)+(dpdv*LP->y);
       cc=V0.z+(dqdu*LP->x)+(dqdv*LP->y);
       bb=-(N.x*aa+N.z*cc+D)/N.y;
      }
      else {                      // projekcia do roviny XY
       if (N.z==0.0f) N.z=1.0f;

       dpdu= den*((V1.x-V2.x)*(T0.y-T2.y)-(V0.x-V2.x)*(T1.y-T2.y));
       dpdv=-den*((V1.x-V2.x)*(T0.x-T2.x)-(V0.x-V2.x)*(T1.x-T2.x));
       dqdu= den*((V1.y-V2.y)*(T0.y-T2.y)-(V0.y-V2.y)*(T1.y-T2.y));
       dqdv=-den*((V1.y-V2.y)*(T0.x-T2.x)-(V0.y-V2.y)*(T1.x-T2.x));

       aa=V0.x+(dpdu*LP->x)+(dpdv*LP->y);
       bb=V0.y+(dqdu*LP->x)+(dqdv*LP->y);
       cc=-(N.x*aa+N.y*bb+D)/N.z;
      }

 // 3D suradnice lumelu
 LP->x=aa; LP->y=bb; LP->z=cc;

 return(0);
}
