#ifndef _MFRONTEND_H_
#define _MFRONTEND_H_

// matematika
#include "m3dmath.h"

// textury
#include "mTexture.h"

// velkost lightmap
#define LM_SIZE 64

// struktura udajov o trojuholniku
typedef struct __tri {
 GLuint id; // identifikator trojuholnika

 tVec3 v0; // 1. vrchol
 tVec3 v1; // 2. vrchol
 tVec3 v2; // 3. vrchol

 tVec2 t0; // 1. texturovacie suradnice
 tVec2 t1; // 2. texturovacie suradnice
 tVec2 t2; // 3. texturovacie suradnice

 GLuint tex1id;  // identifikator 1. textury
 GLuint tex2id;  // identifikator 2. textury
 GLuint LMtexid; // identifikator lightmap textury

 tVec4 c; // farba

 tVec3 n0; // 1. normala
 tVec3 n1; // 2. normala
 tVec3 n2; // 3. normala

 // prvok D rovnice roviny, ktoru tvori trojuholnik,
 // prvkami A,B,C je normala trojuholnika
 GLfloat d;

 tVec4 cn0; // kolizna normala +D 1. steny pre vypocet kolizie
 tVec4 cn1; // kolizna normala +D 2. steny pre vypocet kolizie
 tVec4 cn2; // kolizna normala +D 3. steny pre vypocet kolizie

 // minimalne a maximalne suradnice vrcholov na danych osiach
 // okolo trojuholnika sa tak vytvori pomyselny hranol
 GLfloat minx,maxx,miny,maxy,minz,maxz;

 // indikator spracovania, ak finish_bit=true trojuholnik
 // bol spracovany a nebude sa dalej spracovavat
 bool finish_bit;

 // pomocny indikator, ktorym oznacime trojuholnik,
 // pouziva sa na rozne ucely, najma v pripadoch,
 // ked potrebujeme oznacit vybrany trojuholnik
 bool select;

 // sposob vykreslenia trojuholnika (shader)
 // 0 - klasicky trojuholnik s texturou
 // 1 - trojuholnik predstavujuci vodu
 // 2 - trojuholnik je sucastou kamena
 GLuint type;

 __tri *next; // odkaz na nasledujuci trojuholnik
} _tri;

// pracuje sa s dvoma objektami obsahujucimi trojuholniky:
//
// 1. objekt je zoznam, na ktoreho prvy prvok odkazuje
// premenna first_tri, v tomto zozname su ulozene vsetky
// trojuholniky
//
// 2. objekt je pole aktivnych trojuholnikov, ktore obsahuje
// odkazy na tie trojuholniky z predosleho zoznamu, ktore
// sa maju spracovat a zobrazit na obrazovke, ostatne
// trojuholniky nie su z urciteho dovodu (prechadzanie stromu,
// Frustum Culling) viditelne a nebudu spracovavane

// trieda premennych a funkcii (objektov) pre pracu s OpenGL trojuholnikmi
class mTriangle {
public:
 mTriangle();          // konstruktor
 virtual ~mTriangle(); // destruktor

 _tri *first_tri,*last_tri; // prvy, posledny trojuholnik v zozname
 _tri *tmp_tri,*cur_tri;    // pomocne premenne
 GLuint tri_num;            // celkovy pocet trojuholnikov

 _tri **act_tri;     // pole odkazov na aktivne trojuholniky
 GLuint act_tri_num; // pocet aktivnych trojuholnikov

 _tri **col_tri;     // pole odkazov na kolizne trojuholniky
 GLuint col_tri_num; // pocet koliznych trojuholnikov

 GLuint render_tri;  // pocet zobrazenych trojuholnikov

 // pridanie noveho trojuholnika
 int tri_add(tVec3 V0,tVec3 V1,tVec3 V2,
             tVec2 T0,tVec2 T1,tVec2 T2,
             GLuint TEX1,GLuint TEX2,
             tVec4 C,
             tVec3 N0,tVec3 N1,tVec3 N2,
             GLuint TYPE);
 // vypocet normal v jednotlivych vrcholoch trojuholnika
 void tri_normals(void);
 // vypocet koliznych normal +D jednotlivych stien
 void tri_collision_normals(void);
 // priprava pola odkazov na aktivne trojuholniky
 // a pola odkazov na kolizne trojuholniky
 // je potrebne vykonat po pridani posledneho trojuholnika
 // do zoznamu, dalsie trojuholniky sa uz nebudu pridavat
 int tri_finish(GLuint N);

 // vykreslenie vsetkych trojuholnikov ulozenych v zozname trojuholnikov
 int tri_draw();//mFrustum *F);

 // vymazanie zoznamu trojuholnikov
 int tri_delall(void);

 // vypocet lightmap pre vsetky trojuholniky v zozname trojuholnikov
 int tri_lightmaps(GLfloat LX,GLfloat LY,GLfloat LZ);
private:
 GLuint t_a,t_b,t_c; // pomocne premenne

 tVec3 vec0,vec1,vec2,lp; // pomocne vektory
 tVec2 t0,t1,t2;          // pomocne vektory

 GLuint act_tex_id; // identifikator aktualnej textury

 // 3D suradnice bodu patriaceho lightmape - lumelu
 tVec3 lm_position;
 // pomocne premenne pre vypocet 3D suradnic lumelov
 GLfloat dpdu,dpdv,dqdu,dqdv,den;
 GLfloat aa,bb,cc;
 // pole farieb bodov patriacich lightmape - lumelov
 GLubyte lm_color[3*LM_SIZE*LM_SIZE];

 m3dmath mm;         // pre pracu s matematikou
 mTexture lmTexture; // pre nahratie lightmap textur

 // vypocet 3D suradnic lumelu
 int lm_worldpos(tVec3 *LP,tVec3 V0,tVec3 V1,tVec3 V2,
                           tVec2 T0,tVec2 T1,tVec2 T2,
                           tVec3 N,float D);
};

#endif
