// hlavickove subory *******************************************************
// GL kniznica
#include <GL/gl.h>
// SDL kniznica
#include <SDL/SDL.h>

// vlastny .h subor
#include "mInterface.h"

// konstruktor *************************************************************
mInterface::mInterface()
{
 // rozmery okna
 ViewPort[0]=0; ViewPort[1]=0; ViewPort[2]=0; ViewPort[3]=0;

 lighting_onoff=0;  // osvetlenie
 help_onoff=0;      // napoveda
 pause_motion=0;    // casovo zavisly pohyb
 normals_onoff=0;   // vypnutie vykreslovania normal
 polygon_mode=0;    // vyplnene trojuholniky

 // identifikator pouziteho AVATARa
 Aid=0;

 // pomocne premenne
 t_a=0; t_b=0; t_c=0;

 // stavy OpenGL stavoveho stroja
 ogl_State[0]=0; ogl_State[1]=0;
}

// destruktor **************************************************************
mInterface::~mInterface()
{

}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
int mInterface::ogl_Initialize(void)
{
 // nastavenie 2 vseobecnych materialov
 for (t_a=0;t_a<2;t_a++) {
  ogl_Material[t_a].amb[0]=0.1f; ogl_Material[t_a].amb[1]=0.1f;
  ogl_Material[t_a].amb[2]=0.1f; ogl_Material[t_a].amb[3]=1.0f;
  ogl_Material[t_a].dif[0]=0.8f; ogl_Material[t_a].dif[1]=0.8f;
  ogl_Material[t_a].dif[2]=0.8f; ogl_Material[t_a].dif[3]=1.0f;
  ogl_Material[t_a].spe[0]=1.0f; ogl_Material[t_a].spe[1]=1.0f;
  ogl_Material[t_a].spe[2]=1.0f; ogl_Material[t_a].spe[3]=1.0f;
  ogl_Material[t_a].amb_dif[0]=0.25f; ogl_Material[t_a].amb_dif[1]=0.25f;
  ogl_Material[t_a].amb_dif[2]=0.25f; ogl_Material[t_a].amb_dif[3]=1.0f;
  ogl_Material[t_a].emi[0]=0.0f; ogl_Material[t_a].emi[1]=0.0f;
  ogl_Material[t_a].emi[2]=0.0f; ogl_Material[t_a].emi[3]=1.0f;
  ogl_Material[t_a].shi=128.0f;
  ogl_Material[t_a].transp=0.0f;
  ogl_Material[t_a].sillum[0]=0.0f; ogl_Material[t_a].sillum[1]=0.0f;
  ogl_Material[t_a].sillum[2]=0.0f; ogl_Material[t_a].sillum[3]=1.0f;
 }
 // nastavenie 2 vseobecnych svetiel
 for (t_a=0;t_a<2;t_a++) {
  ogl_Light[t_a].amb[0]=0.1f; ogl_Light[t_a].amb[1]=0.1f;
  ogl_Light[t_a].amb[2]=0.1f; ogl_Light[t_a].amb[3]=1.0f;
  ogl_Light[t_a].dif[0]=0.9f; ogl_Light[t_a].dif[1]=0.9f;
  ogl_Light[t_a].dif[2]=0.9f; ogl_Light[t_a].dif[3]=1.0f;
  ogl_Light[t_a].spe[0]=0.9f; ogl_Light[t_a].spe[1]=0.9f;
  ogl_Light[t_a].spe[2]=0.9f; ogl_Light[t_a].spe[3]=1.0f;
  ogl_Light[t_a].pos[0]=-5.0f; ogl_Light[t_a].pos[1]=5.0f;
  ogl_Light[t_a].pos[2]=0.0f; ogl_Light[t_a].pos[3]=1.0f;
  // pos[3]=1.0f - vsesmerove svetlo, 0.0f - smerove svetlo
  ogl_Light[t_a].spot[0]=0.0f;
  ogl_Light[t_a].spot[1]=0.0f;
  ogl_Light[t_a].spot[2]=-1.0f;
 }

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // nastavenie OpenGL zasobnika farieb a hlbky *****************************
 glClearColor(0.0f,0.0f,0.0f,0.5f); // zasobnik farieb bude vymazavany ciernou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0f

 glDepthFunc(GL_LEQUAL);  // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE); // nastavenie poradia, v ktorom budu definovane vrcholy objektu
 glFrontFace(GL_CCW);     // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT,ogl_Material[0].amb);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,ogl_Material[0].dif);
 glMaterialfv(GL_FRONT,GL_SPECULAR,ogl_Material[0].spe);
 glMaterialfv(GL_FRONT,GL_EMISSION,ogl_Material[0].emi);
 glMaterialf(GL_FRONT,GL_SHININESS,ogl_Material[0].shi);
 // farba bude mat vplyv iba na 'diffuse' komponent materialu
 glColorMaterial(GL_FRONT,GL_DIFFUSE);
 // pre Gourandove svetelne vypocty bude pouzity material priradeny objektu,
 // ktory bude ale stale ovplyvneny farbou zadavanou prikazom glColor
 glEnable(GL_COLOR_MATERIAL);

 // zapnutie gourand-jemneho tienovania
 glShadeModel(GL_SMOOTH);
 // trojuholniky budu vyplnene
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 // zapnutie modulacie textur osvetlenim
 // vysledna farba bodu = farba z textury + osvetlenie
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
 // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
 glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
 glHint(GL_POLYGON_SMOOTH_HINT,GL_NICEST);

 // nastavenie parametrov 1. OpenGL svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_Light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_Light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_Light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);
 glEnable(GL_LIGHT0); // ON
 // nastavenie parametrov 2. OpenGL svetla
 glLightfv(GL_LIGHT1,GL_AMBIENT,ogl_Light[1].amb);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,ogl_Light[1].dif);
 glLightfv(GL_LIGHT1,GL_SPECULAR,ogl_Light[1].spe);
 glLightfv(GL_LIGHT1,GL_POSITION,ogl_Light[1].pos);
 //glEnable(GL_LIGHT1); // OFF
 if (lighting_onoff) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 // svetlo sa pocita len pre privratene strany
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
 // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,1.0f);

 // nastavenie standardnej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 fprintf(stderr,"----------------------------------------------------------\n");

 if (ogl_Texture.LoadCrosshairTexture("data/gfx/crosshair0.bmp")) {
  fprintf(stderr,"Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }

 // nacitanie textury pre podlahu
 if (ogl_Texture.LoadTexture("data/floor.bmp",T_MIPMAP|T_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury pre podlahu!\n"); return(1);
 }
 // nacitanie textury pre kocku
 if (ogl_Texture.LoadTexture("data/crate.bmp",T_MIPMAP|T_TRILINEAR)) {
  fprintf(stderr,"Chyba nahratia textury pre kocku!\n"); return(1);
 }
 // nacitanie textury pre objekt predstavujuci svetelny zdroj
 if (ogl_Texture.LoadTexture("data/flare.tga",T_TRANSPARENT)) {
  fprintf(stderr,"Chyba nahratia textury pre objekt predstavujuci svetelny zdroj!\n"); return(1);
 }

 // priprava zoznamu trojuholnikov - podlaha a kocka
 // vrcholy, normala
 tVec3 v0,v1,v2,n;
 // texturovacie suradnice
 tVec2 t0,t1,t2;
 // farba
 tVec4 c;

 // normaly sa vypocitaju neskor
 n.set(0.0f,0.0f,0.0f);

 // podlaha
 v0.set(-5.0f,0.0f,0.0f); v1.set(5.0f,0.0f,0.0f); v2.set(5.0f,0.0f,-10.0f);
 t0.x=0.0f; t0.y=0.0f; t1.x=1.0f; t1.y=0.0f; t2.x=1.0f; t2.y=1.0f;
 c.x=1.0f; c.y=1.0f; c.z=1.0f; c.w=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[0],0,c,n,n,n,0);
 v0.set(-5.0f,0.0f,0.0f); v1.set(5.0f,0.0f,-10.0f); v2.set(-5.0f,0.0f,-10.0f);
 t0.x=0.0f; t0.y=0.0f; t1.x=1.0f; t1.y=1.0f; t2.x=0.0f; t2.y=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[0],0,c,n,n,n,0);

 // kocka
 v0.set(-1.0f,2.0f,-4.0f); v1.set(1.0f,2.0f,-4.0f); v2.set(1.0f,2.0f,-6.0f);
 t0.x=0.5f; t0.y=0.0f; t1.x=1.0f; t1.y=0.0f; t2.x=1.0f; t2.y=0.5f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);
 v0.set(-1.0f,2.0f,-4.0f); v1.set(1.0f,2.0f,-6.0f); v2.set(-1.0f,2.0f,-6.0f);
 t0.x=0.5f; t0.y=0.0f; t1.x=1.0f; t1.y=0.5f; t2.x=0.5f; t2.y=0.5f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);

 v0.set(-1.0f,0.0f,-6.0f); v1.set(-1.0f,0.0f,-4.0f); v2.set(-1.0f,2.0f,-4.0f);
 t0.x=0.0f; t0.y=0.0f; t1.x=0.5f; t1.y=0.0f; t2.x=0.5f; t2.y=0.5f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);
 v0.set(-1.0f,0.0f,-6.0f); v1.set(-1.0f,2.0f,-4.0f); v2.set(-1.0f,2.0f,-6.0f);
 t0.x=0.0f; t0.y=0.0f; t1.x=0.5f; t1.y=0.5f; t2.x=0.0f; t2.y=0.5f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);

 v0.set(-1.0f,0.0f,-4.0f); v1.set(1.0f,0.0f,-4.0f); v2.set(1.0f,2.0f,-4.0f);
 t0.x=0.0f; t0.y=0.5f; t1.x=0.5f; t1.y=0.5f; t2.x=0.5f; t2.y=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);
 v0.set(-1.0f,0.0f,-4.0f); v1.set(1.0f,2.0f,-4.0f); v2.set(-1.0f,2.0f,-4.0f);
 t0.x=0.0f; t0.y=0.5f; t1.x=0.5f; t1.y=1.0f; t2.x=0.0f; t2.y=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);

 v0.set(1.0f,0.0f,-4.0f); v1.set(1.0f,0.0f,-6.0f); v2.set(1.0f,2.0f,-6.0f);
 t0.x=0.0f; t0.y=0.5f; t1.x=0.5f; t1.y=0.5f; t2.x=0.5f; t2.y=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);
 v0.set(1.0f,0.0f,-4.0f); v1.set(1.0f,2.0f,-6.0f); v2.set(1.0f,2.0f,-4.0f);
 t0.x=0.0f; t0.y=0.5f; t1.x=0.5f; t1.y=1.0f; t2.x=0.0f; t2.y=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);

 v0.set(1.0f,0.0f,-6.0f); v1.set(-1.0f,0.0f,-6.0f); v2.set(-1.0f,2.0f,-6.0f);
 t0.x=0.5f; t0.y=0.5f; t1.x=1.0f; t1.y=0.5f; t2.x=1.0f; t2.y=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);
 v0.set(1.0f,0.0f,-6.0f); v1.set(-1.0f,2.0f,-6.0f); v2.set(1.0f,2.0f,-6.0f);
 t0.x=0.5f; t0.y=0.5f; t1.x=1.0f; t1.y=1.0f; t2.x=0.5f; t2.y=1.0f;
 ogl_Triangle.tri_add(v0,v1,v2,t0,t1,t2,ogl_Texture.tex_id[1],0,c,n,n,n,0);

 // priprava lightmap pre vsetky trojuholniky v zozname
 ogl_Triangle.tri_lightmaps(ogl_Light[0].pos[0],
                            ogl_Light[0].pos[1],
                            ogl_Light[0].pos[2]);

 return(0);
}

// vykreslenie OpenGL sceny ************************************************
int mInterface::ogl_Draw(void)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 // ROTACIA
 glMultMatrixf(Avatar[Aid].matrix);

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 // POSUN X, Y, Z
 glTranslatef(Avatar[Aid].posx,Avatar[Aid].posy,Avatar[Aid].posz);

 // transformacia polohy svetla podla polohy kamery
 //ogl_Light[0].pos[0]=-Avatar[Aid].posx;
 //ogl_Light[0].pos[1]=-Avatar[Aid].posy;
 //ogl_Light[0].pos[2]=-Avatar[Aid].posz;

 // umiestenenie svetla na novu poziciu
 //glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);

 // posun sceny
 glTranslatef(0.0f,-2.0f,-6.0f);

 // vykreslenie vsetkych trojuhonikov zo zoznamu
 ogl_Triangle.tri_draw();

 // vykreslenie svetelneho zdroja
 // zlta farba
 glColor4f(1.0f,1.0f,0.0f,1.0f);
 glEnable(GL_TEXTURE_2D);
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.tex_id[2]);
 // priesvitnost
 glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 glEnable(GL_BLEND);
 glBegin(GL_TRIANGLES);
  glTexCoord2f(0.0f,0.0f); glVertex3f(ogl_Light[0].pos[0],     ogl_Light[0].pos[1],     ogl_Light[0].pos[2]);
  glTexCoord2f(1.0f,0.0f); glVertex3f(ogl_Light[0].pos[0]+0.5f,ogl_Light[0].pos[1],     ogl_Light[0].pos[2]);
  glTexCoord2f(1.0f,1.0f); glVertex3f(ogl_Light[0].pos[0]+0.5f,ogl_Light[0].pos[1]+0.5f,ogl_Light[0].pos[2]);
  glTexCoord2f(0.0f,0.0f); glVertex3f(ogl_Light[0].pos[0],     ogl_Light[0].pos[1],     ogl_Light[0].pos[2]);
  glTexCoord2f(1.0f,1.0f); glVertex3f(ogl_Light[0].pos[0]+0.5f,ogl_Light[0].pos[1]+0.5f,ogl_Light[0].pos[2]);
  glTexCoord2f(0.0f,1.0f); glVertex3f(ogl_Light[0].pos[0],     ogl_Light[0].pos[1]+0.5f,ogl_Light[0].pos[2]);
 glEnd();
 glDisable(GL_BLEND);
 glDisable(GL_TEXTURE_2D);
 // standardna farba
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 // vykreslenie zameriavacieho kriza
 ogl_Texture.DrawCrosshair();

 // vykreslenie napovedy
 if (help_onoff) {

 }
 else {

 }

 return(0);
}

// casovo zavisla uprava objektov, resp. sceny *****************************
int mInterface::ogl_Idle(void)
{
 if (pause_motion==0) {

 }

 return(0);
}

// vyplnene trojuholniky/iba hrany *****************************************
void mInterface::change_polygon_mode(void)
{
 if (polygon_mode==0) {
  polygon_mode=1;
  // budu vykreslene hrany trojuholnikov
  glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
 }
 else {
  polygon_mode=0;
  // trojuholniky budu vyplnene
  glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 }

 // pauza medzi stlaceniami klaves
 SDL_Delay(200);
}

// zmena polohy svetelneho zdroja ******************************************
void mInterface::change_light_pos(GLubyte P)
{
 switch (P) {
  case 0: ogl_Light[0].pos[0]-=1.0f; break; // dolava
  case 1: ogl_Light[0].pos[0]+=1.0f; break; // doprava
  case 2: ogl_Light[0].pos[1]+=1.0f; break; // hore
  case 3: ogl_Light[0].pos[1]-=1.0f; break; // dole
  case 4: ogl_Light[0].pos[2]-=1.0f; break; // dopredu
  case 5: ogl_Light[0].pos[2]+=1.0f; break; // dozadu
  default: break;
 }

 // priprava lightmap pre vsetky trojuholniky v zozname
 // takyto sposob nie je prilis vhodny, pretoze v ramci
 // OpenGL dojde k neustalemu vytvaraniu novych objektov textur,
 // avsak na ukazku to uplne postacuje
 ogl_Triangle.tri_lightmaps(ogl_Light[0].pos[0],
                            ogl_Light[0].pos[1],
                            ogl_Light[0].pos[2]);
}
