#ifndef _MTEXTURE_H_
#define _MTEXTURE_H_

// maximalny pocet vseobecnych textur
#define MAX_TEX_NUM 256

// maximalny pocet lightmap
#define MAX_LM_TEX_NUM 256

// volitelne parametre textury
enum {
 T_DEFAULT=1<<0,
 T_CLAMP=1<<1,
 T_CLAMP_TO_EDGE=1<<2,
 T_TRILINEAR=1<<3,
 T_MIPMAP=1<<4,
 T_TRANSPARENT=1<<5
};

// trieda premennych a funkcii (objektov) pre pracu s OpenGL texturami
class mTexture {
public:
 mTexture();          // konstruktor
 virtual ~mTexture(); // destruktor

 GLuint tex_count;    // pocet nahratych textur
 GLuint lm_tex_count; // pocet lightmap textur

 GLuint tex_id[MAX_TEX_NUM];       // identifikatory nahratych textur
 GLuint lm_tex_id[MAX_LM_TEX_NUM]; // identifikatory lightmap textur

 GLuint cubemap_id;                // identifikator cubemap textury
 GLuint ter_detail_id;             // identifikator detailnej textury terenu
 GLuint crosshair_id;              // identifikator textury zameriavacieho kriza

 int LoadTexture(char *F,GLuint MODE);                           // nahratie 2D textury
 int Load2DTexture(GLuint ID,char *F,GLuint MODE);               // nahratie 2D textury
 int Load1DTexture(GLuint ID,char *F,GLuint MODE);               // nahratie 1D textury
 int LoadBumpTexture(GLuint ID1,GLuint ID2,char *F,GLuint MODE); // nahratie Bump textury
 int LoadCubeMapTexture(char *F1,char *F2,char *F3,              // nahratie CubeMap textury
                        char *F4,char *F5,char *F6,GLuint MODE);
 void DeleteTexture(GLuint ID);                                  // zrusenie OpenGL textury
 int LoadDetailTexture(char *F);    // nahratie detailnej textury terenu
 int LoadCrosshairTexture(char *F); // nahratie textury zameriavacieho kriza
 int DrawCrosshair(void);           // vykreslenie zameriavacieho kriza

 int CreateLightmapTexture(GLuint MODE,GLuint SIZE,GLubyte *DATA); // vytvorenie Lightmap textury
 void DeleteLightmaps(void);                                       // zrusenie Lightmap textur
private:
 GLuint t_a,t_b; // pomocne premenne

 unsigned char *td; // pomocna premenna
 GLint ti;          // pomocna premenna
};

#endif
